type ArrayLength = number | [number, number];
declare class GenerateArray {
    private static readonly _characters;
    private static readonly _charactersWithSpecial;
    /**
     * Create an array from any given value
     *
     * Return values:
     * -null, undefined -> []
     * -Array -> Array (no change)
     * -Function -> [function return value]
     * -Any other value -> [value]
     *
     * @param prototype Value to create an array from
     */
    static from(prototype: any): any[];
    /**
     * Generate an array of the specified length filled with undefined values
     *
     * Example:
     * GenerateArray.blank(5) -> [undefined, undefined, undefined, undefined, undefined]
     *
     * @param length Size of array: a set length or a range [min, max] (both inclusive, random length will be chosen)
     */
    static blank(length: ArrayLength): any[];
    /**
     * Generate an array of the specified length filled with the specified value
     *
     * Example:
     * GenerateArray.uniform(5, 7) -> [7, 7, 7, 7, 7]
     *
     * @param length Size of array (or an array range [min, max])
     * @param value Value to fill array with
     */
    static uniform(length: ArrayLength, value: any): any[];
    /**
     * Generate an array of the specified length filled with values generated by the given function
     *
     * Examples:
     * GenerateArray.custom(() => Math.random(), 5) -> [0.12345, 0.6789, 0.101112, 0.131415, 0.161718] (possible values)
     * GenerateArray.custom(() => Math.floor(Math.random() * 10), 6) -> [7, 2, 5, 9, 1, 3] (possible values)
     *
     * @param generator Function to generate each array value
     * @param length Size of array (or an array range [min, max])
     */
    static custom: (generator: () => any, length: ArrayLength) => any[];
    /**
     * Generate an array of numbers from the start value up to a max/min value (both inclusive) with the given step value
     *
     * Examples:
     * GenerateArray.counting(0, 5) -> [0, 1, 2, 3, 4, 5]
     * GenerateArray.counting(1, 10, 2) -> [1, 3, 5, 7, 9]
     * GenerateArray.counting(1.2, 37.9, 4.7) -> [1.2, 5.9, 10.6, 15.3, 20, 24.7, 29.4, 34.1]
     * GenerateArray.counting(10, 1, 2, true) -> [10, 8, 6, 4, 2]
     *
     * @param start Number to start at
     * @param end Value to stop at (may not be included in resulting array)
     * @param step Value added at each step (default 1). Can be negative is end < start
     */
    static counting: (start: number, end: number, step?: number) => number[];
    /**
     * Generate an array of random integers of the specified length within the given range
     *
     * Examples:
     * GenerateArray.integers(6) -> [23, 59, 2, 91, 45, 21] (possible values)
     * GenerateArray.integers(5, 10, 20) -> [15, 17, 12, 19, 11] (possible values)
     *
     * @param length Size of array >= 1 (or an array range [min, max])
     * @param min Minimum value (inclusive), default 0
     * @param max Maximum value (exclusive), default 100
     */
    static integers: (length: ArrayLength, min?: number, max?: number) => number[];
    /**
     * Generate an array of random decimals of the specified length within the given range. May sometimes produce
     * integers if the range is significantly large
     *
     * Examples:
     * GenerateArray.decimals(5) -> [0.12345, 0.6789, 0.101112, 0.131415, 0.161718] (possible values)
     * GenerateArray.decimals(6, 0, 10) -> [7.12345, 2.6789, 5.101112, 9.131415, 1.161718, 3.192021] (possible values)
     *
     * @param length Size of array >= 1 (or an array range [min, max])
     * @param min Minimum value (inclusive), default 0
     * @param max Maximum value (exclusive), default 1
     */
    static decimals: (length: ArrayLength, min?: number, max?: number) => number[];
    /**
     * Generate an array of random strings of the specified length within the given range
     *
     * @param length Size of array >= 1 (or an array range [min, max])
     * @param minLength Minimum length of string (>= 1), default 1
     * @param maxLength Maximum length of string (>= minLength), default 10
     * @param specialChars If true, includes special characters (!@#$%^&*()_+/\{}[]|;:'",.<>?`~) in the generated
     * strings. By default, only letters and numbers are used (a-zA-Z0-9)
     */
    static strings: (length: ArrayLength, minLength?: number, maxLength?: number, specialChars?: boolean) => string[];
    /**
     * Generate an array of the specified length. Each value is obtained from a random generator with equal chances
     *
     * Example:
     * CustomArray.random(5, [() => Math.random(), () => Math.floor(Math.random() * 10)]) -> [0.12345, 7, 0.101112, 1, 3] (possible values)
     *
     * @param length Size of array to be generated >= 1 (or an array range [min, max])
     * @param generators Array of functions that generate a value (value can be anything)
     */
    static generators: (length: ArrayLength, generators: (() => any)[]) => any[];
    /**
     * Generate array of the specified length with the specified generators. Each generator has a weighted chance of
     * being used for each element in the array
     *
     * Example:
     * CustomArray.randomChance(7, [{ generator: () => Math.random(), chance: 0.3 }, { generator: () => Math.floor(Math.random() * 10), chance: 0.7 }]) -> [4, 0.12345, 7, 0.101112, 7, 1, 8] (possible values)
     *
     * @param length Size of array to be generated >= 1 (or an array range [min, max])
     * @param generators Array of objects in the form { generator: () => any, chance: number }. Chance for all
     * generators combined must add up to 1
     */
    static weightedGenerators: (length: ArrayLength, generators: {
        generator: () => any;
        chance: number;
    }[]) => any[];
    /**
     * Generates an array of the specified length with the specified generators. Each generator is called a fixed
     * amount of times
     *
     * @param generators Array of objects in the form { generator: () => any, count: number }. Generators must
     * return a value (any value), and the count of all generators combined must equal the length of the array
     * @param random If true (default), the order of the generated values is random. If false, all values from a
     * specific generator are next to each other in a sequence
     */
    static fixedCountGenerators: (generators: {
        generator: () => any;
        count: number;
    }[], random?: boolean) => any[];
    private static _deepArray;
    /**
     * Generates an multidimensional array of the specific length (number of arrays in each array) and depth (how many
     * levels of arrays to generate before stopping) where the base array is empty.
     *
     * Examples:
     * GenerateArray.emptyND(1, 2) -> [[]] (Each array has 1 array in it, and there are 2 levels of arrays)
     * GenerateArray.emptyND(2, 2) -> [[], []] (Each array has 2 arrays in it, and there are 2 levels of arrays)
     * GenerateArray.emptyND(1, 3) -> [[[]]] (There is 1 array in each array, and 3 levels of arrays)
     * GenerateArray.emptyND(2, 3) -> [[[], []], [[], []]] (There are 2 arrays in each array, and 3 levels of arrays)
     *
     * @param length Number of arrays in each array (>= 1, or an array range [min, max]), excluding the base array
     * (array at the lowest depth)
     * @param depth Dimension of the returned array (>= 2), i.e. how many layers of arrays there are. Dept of 1 is
     * just [] regardless of length, so not allowed to reduce confusion
     */
    static emptyND: (length: ArrayLength, depth: number) => any[];
    /**
     * Generates a multidimensional array of the specific length (number of arrays in each array) and depth (how many
     * levels of arrays to generate before stopping) where all base arrays are filled with the given value(s).
     *
     * If the value is an array, the base arrays will be filled with the values of the array instead of adding an
     * extra depth of arrays, e.g. 7 -> [7], [1, 2, 3] -> [1, 2, 3] (put the array in an array if you'd like this)
     *
     * Examples:
     * GenerateArray.uniformND(7, 3, 2) -> [[7], [7], [7]]
     * GenerateArray.uniformND([1, 2, 3], 3, 2) -> [[1, 2, 3], [1, 2, 3], [1, 2, 3]]
     *
     * @param value Value of the base array. If not an array, the base array will be an array filled with this value.
     * @param length Number of arrays in each array (>= 1, or an array range [min, max]), excluding the base array (array at the lowest depth)
     * @param depth Dimension of the returned array (>= 2), i.e. how many layers of arrays there are. Depth of 1
     * isn't included to reduce confusion, as it would just return the value in an array (or the value if it's an array)
     * e.g. MultidimensionalArray.uniform([1, 2, 3], 3, 1) -> [1, 2, 3]
     */
    static uniformND: (value: any, length: ArrayLength, depth: number) => any[];
    /**
     * Generates a multidimensional array of the specific length (number of arrays in each array) and depth (how many
     * levels of arrays to generate before stopping) where the base arrays are generated by the given function.
     *
     * Examples:
     * GenerateArray.customND(() => Math.floor(Math.random() * 10), 3, 2) -> [[7], [2], [5]] (possible output)
     *
     * @param generator Function to generate the base array. Note this function is called once for each base array,
     * so to have multiple values, this function should return an array of multiple values (e.g.
     * GenerateArray.custom(() => [1, 2, 3], len, depth))
     * @param length Number of arrays in each array (>= 1, or an array range [min, max]), excluding the base array (array
     * at the lowest depth)
     * @param depth Dimension of the returned array (>= 2), i.e. how many layers of arrays there are. Depth of 1 isn't
     * allowed to reduce confusion as it would just put the value of one function call in an array, e.g. () => 7 -> [7]
     * (to get a 1-D array with multiple values, use GenerateArray.custom(() => ..., len) instead)
     */
    static customND: (generator: () => any[], length: ArrayLength, depth: number) => any[];
}

/**
 * Generator for a random integer. By default, the number is between 0 and 100 (both inclusive)
 *
 * @param min Minimum value
 * @param max Maximum value
 */
declare const integer: (min?: number, max?: number) => () => number;
/**
 * Generator for a random decimal number. By default, the number is between 0 and 100 with a precision of 5
 *
 * @param min Minimum value
 * @param max Maximum value (must be >= min)
 * @param precision Number of decimal places (note: if the last decimal place(s) are zeros, they won't be displayed
 * in the return value as it's a number)
 */
declare const decimal: (min?: number, max?: number, precision?: number) => () => number;
/**
 * Generator for a random string. By default, the string is a length of 10 and uses alphanumeric characters
 *
 * @param length Length of the string as a set number or a range [min, max] (both inclusive)
 * @param specialChars True to include special characters
 */
declare const string: (length?: number | [number, number], specialChars?: boolean) => () => string;
/**
 * Generator for a random boolean value
 *
 * @param trueChance Chance for the value to be true (between 0 and 1), default is 0.5
 */
declare const boolean: (trueChance?: number) => () => boolean;
/**
 * Generator for a random date. By default, the date is between 1970-01-01 and the current date
 *
 * @param min Lower bound for the date, either a Date object or number of milliseconds since epoch (1970-01-01 00:00:00)
 * @param max Upper bound for the date, either a Date object or number of milliseconds since epoch (1970-01-01 00:00:00)
 */
declare const date: (min?: Date | number, max?: Date | number) => () => Date;
/**
 * Generator for a random phone number
 *
 * @param countryCode Generates a random country code each time if true
 * @param format Whether to format the phone number, e.g. +1 (123)-456-7890. If false (default), string is just numbers
 */
declare const phone: (countryCode?: boolean, format?: boolean) => () => string;
/**
 * Generator for a random UUID
 */
declare const uuid: () => () => string;
/**
 * Generator for a random IP address (IPv6 by default)
 *
 * @param IPv6 Set to true to generate an IPv6 address instead of IPv4
 * @param mask Set to true to generate an IP address with a subnet mask (/x, 0 <= x <= 32 at the end)
 */
declare const ipAddress: (IPv6?: boolean, mask?: boolean) => () => string;
/**
 * Generator for a random email address. By default, the username is a name e.g. john.smith@gmail.com
 *
 * @param rngUsername Generate a username of random characters each time (e.g. qGTcds9fx@protonmail.com), set this
 * parameter to the length (or length range) that the username should be
 */
declare const email: (rngUsername?: number | [number, number]) => () => string;
/**
 * Generator for a random URL
 *
 * @param protocol Set to false to generate a URL without the https://www. protocol (e.g. example.com)
 */
declare const url: (protocol?: boolean) => () => string;
/**
 * Generator for a random name
 */
declare const name: () => () => string;

declare class ArrayUtils {
    /**
     * Flattens the array to a single dimension. This removes all nested arrays, placing all non-array elemnts in a single array
     *
     * Example:
     * flatten([1, [2, 3], [4, [5, 6]]]) -> [1, 2, 3, 4, 5, 6]
     *
     * @param arr
     */
    static flatten(arr: any[]): any[];
    /**
     * Adds the specified number of dimensions to the array (i.e. wraps the array in arrays the specified number of times)
     *
     * Example:
     * addDimensions(arr, 1) -> [arr]
     * addDimensions(arr, 2) -> [[arr]]
     * addDimensions(arr, 4) -> [[[[arr]]]]
     *
     * @param arr Array to add dimensions to
     * @param depth Number of dimensions to add (>= 1)
     */
    static addDimensions<T>(arr: T[], depth: number): T[];
    /**
     * Multiplies the array within itself n times (i.e. repeats the elements in the array n times). Does not modify the original array
     *
     * Example:
     * multiplyLength([1, 2], 3) -> [1, 2, 1, 2, 1, 2]
     * multiplyLength([1, 2], 3, true) -> [1, 1, 1, 2, 2, 2]
     *
     * @param arr Array to multiply
     * @param factor Number of times to multiply the array (>= 2)
     * @param elementWise If false (default), repeats the entire array before repeating the next array. If true,
     * repeats each element n times before repeating the next element in the array
     */
    static multiplyLength(arr: any[], factor: number, elementWise?: boolean): any[];
    /**
     * Returns a copy of the given array with any duplicate values removed
     *
     * @param arr Array to remove duplicates from
     */
    static removeDuplicates(arr: any[]): any[];
}

declare class GenerateArrayError extends Error {
    constructor(message: string);
}

export { ArrayUtils, GenerateArray, GenerateArrayError, boolean, date, decimal, email, integer, ipAddress, name, phone, string, url, uuid };
